#include "SceneQuery.h"
#include "glInterface.h"
#include "Vector3.h"
#include "Camera.h"
#include "SceneQuery.h"
#include "RenderTraverser.h"


void startil();
void stopil();


namespace CHCDemoEngine
{

using namespace std;


const static int viewport[4] = {0, 0, 512, 512};
//const static int viewport[4] = {0, 0, 1024, 768};



SceneQuery::SceneQuery(const AxisAlignedBox3 &sceneBox, RenderTraverser *renderer): 
mSceneBox(sceneBox), mDepth(NULL)
{
	Prepare(renderer);
}


bool SceneQuery::CalcIntersection(Vector3 &pt)
{
	int px = (pt.x - mSceneBox.Min(0)) * (viewport[2] - 1) / mSceneBox.Size(0);
	int py = (pt.y - mSceneBox.Min(1)) * (viewport[3] - 1) / mSceneBox.Size(1);

	const float d = mDepth[px + py * viewport[1]];

	if (d > 0.0f)
	{
		pt.z =  mSceneBox.Max().z - mSceneBox.Size().z * d;
		cout << "new depth " << pt.z << " (" << d << ")" << endl;

		return true;
	}

	return false;
}


void SceneQuery::Prepare(RenderTraverser *renderer)
{
	cout << "Preparing scene queries" << endl;

	const float xlen = mSceneBox.Size().x * 0.5f;
	const float ylen = mSceneBox.Size().y * 0.5f;
	
	Camera *orthoCam = new Camera(xlen, ylen);
	orthoCam->SetOrtho(true);

	orthoCam->SetNear(0.0f);
	orthoCam->Yaw(M_PI * 0.5f);

	Vector3 pos = Vector3(mSceneBox.Center().x, mSceneBox.Center().y, mSceneBox.Max().z);
	orthoCam->SetPosition(pos);

	//glPixelStorei(GL_PACK_ROW_LENGTH, viewport[2]);

	// hack: should create offscreen buffer for this
	glViewport(0, 0, viewport[2], viewport[3]);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	glOrtho(-xlen, xlen, -ylen, ylen, 0.0f, mSceneBox.Size().z); 
	
	glMatrixMode(GL_MODELVIEW);
	
	orthoCam->SetupCameraView();

	glClear(GL_DEPTH_BUFFER_BIT);

	mDepth = new float[viewport[2] * viewport[3]];

	//renderer->SetCamera(orthoCam);

	renderer->RenderScene();

	glReadPixels(0, 0, viewport[2], viewport[3], GL_DEPTH_COMPONENT, GL_FLOAT, mDepth);

	DEL_PTR(orthoCam);
}

}